@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3F.cxx,v 1.3 2002/05/14 15:22:28 stephena Exp $
+// $Id: Cart3F.cxx,v 1.12 2005/12/17 01:23:07 stephena Exp $
 //============================================================================
 
 #include <assert.h>
 #include "Cart3F.hxx"
 #include "System.hxx"
+#include "TIA.hxx"
 #include "Serializer.hxx"
 #include "Deserializer.hxx"
 #include <iostream>
@@ -67,8 +68,9 @@ void Cartridge3F::install(System& system)
   assert((0x1800 & mask) == 0);
 
   // Set the page accessing methods for the hot spots (for 100% emulation
-  // I would need to chain any accesses below 0x40 to the TIA but for
-  // now I'll just forget about them)
+  // we need to chain any accesses below 0x40 to the TIA. Our poke() method
+  // does this via mySystem->tiaPoke(...), at least until we come up with a
+  // cleaner way to do it.)
   System::PageAccess access;
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
   {
@@ -116,11 +118,30 @@ void Cartridge3F::poke(uInt16 address, uInt8 value)
   {
     bank(value);
   }
+
+  mySystem->tia().poke(address, value);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge3F::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  if(address < 0x0800)
+  {
+    myImage[(address & 0x07FF) + myCurrentBank * 2048] = value;
+  }
+  else
+  {
+    myImage[(address & 0x07FF) + mySize - 2048] = value;
+  }
+  return true;
+} 
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3F::bank(uInt16 bank)
 { 
+  if(bankLocked) return;
+
   // Make sure the bank they're asking for is reasonable
   if((uInt32)bank * 2048 < mySize)
   {
@@ -149,6 +170,17 @@ void Cartridge3F::bank(uInt16 bank)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int Cartridge3F::bank() {
+  return myCurrentBank;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int Cartridge3F::bankCount() {
+  return mySize/2048;
+}
+
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge3F::save(Serializer& out)
 {
@@ -157,7 +189,7 @@ bool Cartridge3F::save(Serializer& out)
   try
   {
     out.putString(cart);
-    out.putLong(myCurrentBank);
+    out.putInt(myCurrentBank);
   }
   catch(char *msg)
   {
@@ -183,7 +215,7 @@ bool Cartridge3F::load(Deserializer& in)
     if(in.getString() != cart)
       return false;
 
-    myCurrentBank = (uInt16) in.getLong();
+    myCurrentBank = (uInt16) in.getInt();
   }
   catch(char *msg)
   {
@@ -201,3 +233,9 @@ bool Cartridge3F::load(Deserializer& in)
 
   return true;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* Cartridge3F::getImage(int& size) {
+  size = mySize;
+  return &myImage[0];
+}
