@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart4K.cxx,v 1.3 2002/05/14 15:22:28 stephena Exp $
+// $Id: Cart4K.cxx,v 1.7 2005/07/30 16:25:48 urchlay Exp $
 //============================================================================
 
 #include <assert.h>
@@ -83,6 +83,13 @@ void Cartridge4K::poke(uInt16, uInt8)
   // This is ROM so poking has no effect :-)
 } 
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge4K::patch(uInt16 address, uInt8 value)
+{
+	myImage[address & 0x0FFF] = value;
+	return true;
+} 
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge4K::save(Serializer& out)
 {
@@ -129,3 +136,9 @@ bool Cartridge4K::load(Deserializer& in)
 
   return true;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* Cartridge4K::getImage(int& size) {
+  size = 4096;
+  return &myImage[0];
+}
