@@ -8,18 +8,21 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE7.cxx,v 1.2 1998/07/15 20:30:55 bwmott Exp $
+// $Id: CartE7.cxx,v 1.13 2005/12/17 01:23:07 stephena Exp $
 //============================================================================
 
 #include <assert.h>
 #include "CartE7.hxx"
 #include "Random.hxx"
 #include "System.hxx"
+#include "Serializer.hxx"
+#include "Deserializer.hxx"
+#include <iostream>
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeE7::CartridgeE7(const uInt8* image)
@@ -31,7 +34,7 @@ CartridgeE7::CartridgeE7(const uInt8* image)
   }
 
   // Initialize RAM with random values
-  Random random;
+  class Random random;
   for(uInt32 i = 0; i < 2048; ++i)
   {
     myRAM[i] = random.next();
@@ -134,9 +137,20 @@ void CartridgeE7::poke(uInt16 address, uInt8)
   // way page accessing has been setup
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeE7::patch(uInt16 address, uInt8 value)
+{
+	address = address & 0x0FFF;
+	myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)] = value;
+	bank(myCurrentSlice[0]);
+	return true;
+} 
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::bank(uInt16 slice)
 { 
+  if(bankLocked) return;
+
   // Remember what bank we're in
   myCurrentSlice[0] = slice;
   uInt16 offset = slice << 11;
@@ -181,6 +195,16 @@ void CartridgeE7::bank(uInt16 slice)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeE7::bank() {
+  return myCurrentSlice[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeE7::bankCount() {
+  return 8;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::bankRAM(uInt16 bank)
 { 
@@ -212,3 +236,85 @@ void CartridgeE7::bankRAM(uInt16 bank)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeE7::save(Serializer& out)
+{
+  string cart = name();
+
+  try
+  {
+    uInt32 i;
+
+    out.putString(cart);
+
+    out.putInt(2);
+    for(i = 0; i < 2; ++i)
+      out.putInt(myCurrentSlice[i]);
+
+    out.putInt(myCurrentRAM);
+
+    // The 2048 bytes of RAM
+    out.putInt(2048);
+    for(i = 0; i < 2048; ++i)
+      out.putInt(myRAM[i]);
+  }
+  catch(char *msg)
+  {
+    cerr << msg << endl;
+    return false;
+  }
+  catch(...)
+  {
+    cerr << "Unknown error in save state for " << cart << endl;
+    return false;
+  }
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeE7::load(Deserializer& in)
+{
+  string cart = name();
+
+  try
+  {
+    if(in.getString() != cart)
+      return false;
+
+    uInt32 i, limit;
+
+    limit = (uInt32) in.getInt();
+    for(i = 0; i < limit; ++i)
+      myCurrentSlice[i] = (uInt16) in.getInt();
+
+    myCurrentRAM = (uInt16) in.getInt();
+
+    // The 2048 bytes of RAM
+    limit = (uInt32) in.getInt();
+    for(i = 0; i < limit; ++i)
+      myRAM[i] = (uInt8) in.getInt();
+  }
+  catch(char *msg)
+  {
+    cerr << msg << endl;
+    return false;
+  }
+  catch(...)
+  {
+    cerr << "Unknown error in load state for " << cart << endl;
+    return false;
+  }
+
+  // Set up the previously used banks for the RAM and segment
+  bankRAM(myCurrentRAM);
+  bank(myCurrentSlice[0]);
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeE7::getImage(int& size) {
+  size = 16384;
+  return &myImage[0];
+}
