@@ -8,18 +8,21 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartMC.cxx,v 1.1 1998/09/20 21:27:01 bwmott Exp $
+// $Id: CartMC.cxx,v 1.9 2005/12/17 01:23:07 stephena Exp $
 //============================================================================
 
 #include <assert.h>
 #include "CartMC.hxx"
 #include "Random.hxx"
 #include "System.hxx"
+#include "Serializer.hxx"
+#include "Deserializer.hxx"
+#include <iostream>
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size)
@@ -34,7 +37,7 @@ CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size)
   myRAM = new uInt8[32 * 1024];
 
   // Initialize RAM with random values
-  Random random;
+  class Random random;
   for(i = 0; i < 32 * 1024; ++i)
   {
     myRAM[i] = random.next();
@@ -216,3 +219,86 @@ void CartridgeMC::poke(uInt16 address, uInt8 value)
   }  
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::patch(uInt16 address, uInt8 value)
+{
+	// TODO: implement
+	return false;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::save(Serializer& out)
+{
+  uInt32 i;
+  string cart = name();
+
+  try
+  {
+    out.putString(cart);
+
+    // The currentBlock array
+    out.putInt(4);
+    for(i = 0; i < 4; ++i)
+      out.putInt(myCurrentBlock[i]);
+
+    // The 32K of RAM
+    out.putInt(32 * 1024);
+    for(i = 0; i < 32 * 1024; ++i)
+      out.putInt(myRAM[i]);
+  }
+  catch(char *msg)
+  {
+    cerr << msg << endl;
+    return false;
+  }
+  catch(...)
+  {
+    cerr << "Unknown error in save state for " << cart << endl;
+    return false;
+  }
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::load(Deserializer& in)
+{
+  uInt32 i;
+  string cart = name();
+
+  try
+  {
+    uInt32 limit;
+
+    if(in.getString() != cart)
+      return false;
+
+    // The currentBlock array
+    limit = (uInt32) in.getInt();
+    for(i = 0; i < limit; ++i)
+      myCurrentBlock[i] = (uInt8) in.getInt();
+
+    // The 32K of RAM
+    limit = (uInt32) in.getInt();
+    for(i = 0; i < limit; ++i)
+      myRAM[i] = (uInt8) in.getInt();
+  }
+  catch(char *msg)
+  {
+    cerr << msg << endl;
+    return false;
+  }
+  catch(...)
+  {
+    cerr << "Unknown error in load state for " << cart << endl;
+    return false;
+  }
+
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeMC::getImage(int& size) {
+  size = 128 * 1024; // FIXME: keep track of original size
+  return &myImage[0];
+}
