@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Switches.cxx,v 1.2 1998/07/15 20:51:28 bwmott Exp $
+// $Id: Switches.cxx,v 1.5 2006/03/05 01:18:42 stephena Exp $
 //============================================================================
 
 #include "Event.hxx"
@@ -25,7 +25,7 @@ Switches::Switches(const Event& event, const Properties& properties)
     : myEvent(event),
       mySwitches(0xFF)
 {
-  if(properties.get("Console.RightDifficulty") == "B")
+  if(properties.get(Console_RightDifficulty) == "B")
   {
     mySwitches &= ~0x80;
   }
@@ -34,7 +34,7 @@ Switches::Switches(const Event& event, const Properties& properties)
     mySwitches |= 0x80;
   }
 
-  if(properties.get("Console.LeftDifficulty") == "B")
+  if(properties.get(Console_LeftDifficulty) == "B")
   {
     mySwitches &= ~0x40;
   }
@@ -43,7 +43,7 @@ Switches::Switches(const Event& event, const Properties& properties)
     mySwitches |= 0x40;
   }
 
-  if(properties.get("Console.TelevisionType") == "Color")
+  if(properties.get(Console_TelevisionType) == "COLOR")
   {
     mySwitches |= 0x08;
   }
@@ -72,20 +72,20 @@ uInt8 Switches::read()
 
   if(myEvent.get(Event::ConsoleRightDifficultyA) != 0)
   {
-    mySwitches &= ~0x80;
+    mySwitches |= 0x80;
   }
   else if(myEvent.get(Event::ConsoleRightDifficultyB) != 0) 
   {
-    mySwitches |= 0x80;
+    mySwitches &= ~0x80;
   }
 
   if(myEvent.get(Event::ConsoleLeftDifficultyA) != 0)
   {
-    mySwitches &= ~0x40;
+    mySwitches |= 0x40;
   }
   else if(myEvent.get(Event::ConsoleLeftDifficultyB) != 0)
   {
-    mySwitches |= 0x40;
+    mySwitches &= ~0x40;
   }
 
   if(myEvent.get(Event::ConsoleSelect) != 0)
